@@ -392,7 +392,7 @@ def write_input_file(dim, nodes, elements, element_type, element_sets_dict, outp
     outfile.close()
 
 
-def mesh(gap, dimension, node_shape, element_type, model_path, output_path, status):
+def mesh(gap, dimension, node_shape, element_type, model_path, output_path, status, is_interface=True):
     model_file = os.path.join(model_path, 'model.npy')
     circles = np.load(model_file)
     dim = circles.shape[-1]-1
@@ -422,13 +422,36 @@ def mesh(gap, dimension, node_shape, element_type, model_path, output_path, stat
         elements = [element_index, n1, n2, n3, n4]
 
         element_sets = np.zeros(len(element_index))
-
-        for i, circle in enumerate(circles):
-            cx = circle[0]
-            cy = circle[1]
-            r = circle[2] - gap
-            index = (ecx - cx)**2 + (ecy - cy)**2 < r**2
-            element_sets[index] = int(i) + 1
+            
+        n1x = x[n1]
+        n1y = y[n1]
+        n2x = x[n2]
+        n2y = y[n2]
+        n3x = x[n3]
+        n3y = y[n3]
+        n4x = x[n4]
+        n4y = y[n4]
+        
+        if not is_interface:
+            for i, circle in enumerate(circles):
+                cx = circle[0]
+                cy = circle[1]
+                r = circle[2] - gap
+                index = (ecx - cx)**2 + (ecy - cy)**2 < r**2
+                element_sets[index] = int(i) + 1
+        else:
+            for i, circle in enumerate(circles):
+                cx = circle[0]
+                cy = circle[1]
+                r = circle[2] - gap
+                index1 = (n1x-cx)**2 + (n1y-cy)**2 < r**2
+                index2 = (n2x-cx)**2 + (n2y-cy)**2 < r**2
+                index3 = (n3x-cx)**2 + (n3y-cy)**2 < r**2
+                index4 = (n4x-cx)**2 + (n4y-cy)**2 < r**2
+                index_inner = index1 & index2 & index3 & index4
+                element_sets[index_inner] = int(i) + 1
+                index_inter = (index1 | index2 | index3 | index4) ^ index_inner
+                element_sets[index_inter] = -1
 
     elif dim >= 3:
         node_index = create_node_index(node_shape)
@@ -461,14 +484,58 @@ def mesh(gap, dimension, node_shape, element_type, model_path, output_path, stat
         elements = [element_index, n1, n2, n3, n4, n5, n6, n7, n8]
         element_sets = np.zeros(len(element_index))
 
-        for i, circle in enumerate(circles):
-            cx = circle[0]
-            cy = circle[1]
-            cz = circle[2]
-            r = circle[-1] - gap
-            index = (ecx - cx)**2 + (ecy - cy)**2 + (ecz-cz)**2 < r**2
-            element_sets[index] = int(i) + 1
-
+        n1x = x[n1]
+        n1y = y[n1]
+        n1z = z[n1]
+        n2x = x[n2]
+        n2y = y[n2]
+        n2z = z[n2]
+        n3x = x[n3]
+        n3y = y[n3]
+        n3z = z[n3]
+        n4x = x[n4]
+        n4y = y[n4]
+        n4z = z[n4]
+        n5x = x[n5]
+        n5y = y[n5]
+        n5z = z[n5]
+        n6x = x[n6]
+        n6y = y[n6]
+        n6z = z[n6]
+        n7x = x[n7]
+        n7y = y[n7]
+        n7z = z[n7]
+        n8x = x[n8]
+        n8y = y[n8]
+        n8z = z[n8]
+        
+        if not is_interface:
+            for i, circle in enumerate(circles):
+                cx = circle[0]
+                cy = circle[1]
+                cz = circle[2]
+                r = circle[-1] - gap
+                index = (ecx - cx)**2 + (ecy - cy)**2 + (ecz-cz)**2 < r**2
+                element_sets[index] = int(i) + 1
+        else:
+            for i, circle in enumerate(circles):
+                cx = circle[0]
+                cy = circle[1]
+                cz = circle[2]
+                r = circle[-1] - gap
+                index1 = (n1x-cx)**2 + (n1y-cy)**2 + (n1z-cz)**2 < r**2
+                index2 = (n2x-cx)**2 + (n2y-cy)**2 + (n2z-cz)**2 < r**2
+                index3 = (n3x-cx)**2 + (n3y-cy)**2 + (n3z-cz)**2 < r**2
+                index4 = (n4x-cx)**2 + (n4y-cy)**2 + (n4z-cz)**2 < r**2
+                index5 = (n5x-cx)**2 + (n5y-cy)**2 + (n5z-cz)**2 < r**2
+                index6 = (n6x-cx)**2 + (n6y-cy)**2 + (n6z-cz)**2 < r**2
+                index7 = (n7x-cx)**2 + (n7y-cy)**2 + (n7z-cz)**2 < r**2
+                index8 = (n8x-cx)**2 + (n8y-cy)**2 + (n8z-cz)**2 < r**2
+                index_inner = index1 & index2 & index3 & index4 & index5 & index6 & index7 & index8
+                element_sets[index_inner] = int(i) + 1
+                index_inter = (index1 | index2 | index3 | index4 | index5 | index6 | index7 | index8) ^ index_inner
+                element_sets[index_inter] = -1
+                
     element_sets_names = np.unique(element_sets)
 
     element_sets_dict = {}
@@ -476,15 +543,18 @@ def mesh(gap, dimension, node_shape, element_type, model_path, output_path, stat
         element_sets_dict[int(name)] = []
         element_sets_dict['MATRIX'] = []
         element_sets_dict['PARTICLES'] = []
+        element_sets_dict['INTERFACES'] = []
         element_sets_dict['ALL'] = []
     for i, _ in enumerate(element_sets):
         element_sets_dict[int(element_sets[i])].append(element_index[i]+1)
         if int(element_sets[i]) == 0:
             element_sets_dict['MATRIX'].append(element_index[i]+1)
+        elif int(element_sets[i]) == -1:
+            element_sets_dict['INTERFACES'].append(element_index[i]+1)
         else:
             element_sets_dict['PARTICLES'].append(element_index[i]+1)
         element_sets_dict['ALL'].append(element_index[i]+1)
-
+        
     write_input_file(dim, nodes, elements, element_type, element_sets_dict, output_path)
 
     status['message']['element_sets_names'] = element_sets_names.tolist()
@@ -519,10 +589,10 @@ def create_mesh(gap, size, dimension, node_shape, element_type, model_path, outp
 
 if __name__ == "__main__":
     gap = 0.0
-    size = [[0, 0.1], [0, 0.1]]
+    size = [[0.0, 0.125], [0.0, 0.125], [0.0, 0.125]]
     dimension = [s[1] for s in size]
-    node_shape = [100, 100]
-    element_type = 'CPE4T'
+    node_shape = [65, 65, 65]
+    element_type = 'C3D8T'
     model_path = ''
     output_path = ''
     status = {'status': 'Submit', 'log': '', 'progress': 0}
